#include <assert.h>

typedef unsigned int uint32_t;
typedef unsigned short uint16_t;
typedef unsigned char uint8_t;

#define MAX_RECORDS  64
uint16_t numberOfRecords;
uint16_t records[MAX_RECORDS];

#define MAX_BUFFER  64
uint8_t msgBuffer[MAX_BUFFER];

uint32_t intervalCounter; 
uint16_t tickCounter;     
uint16_t interval;     
uint32_t startTime;

#define IDLE 0
#define LOGGING 1
uint8_t systemState;

#define CMD_DEC_OK 1
#define CMD_DEC_ERROR 0
#define RSP_SUCCESS 1
#define RSP_ERROR 0

#define TASK_COMMUNICATE 1
#define TASK_MEASURE 2

#define CMD_START 1
#define CMD_STOP 2
#define CMD_READ_STATE 3
#define CMD_READ_DATA 4

// Lihao
const int __CPROVER_thread_priorities[] = {5, 22};
const char* __CPROVER_threads[] = {"c::task_communicate", "c::task_measure"};

//void irq_err(void) { ERROR: goto ERROR;}
void task_communicate(void);
void task_measure(void);

extern _Bool  __VERIFIER_nondet_bool();
extern int __VERIFIER_nondet_int();
extern void __VERIFIER_assume(int);

extern void __VERIFIER_error() __attribute__ ((__noreturn__));
void __VERIFIER_assert(int cond) { if(!(cond)) { ERROR: __VERIFIER_error(); } }

/*void __VERIFIER_assert(int x)
{
  if(!x)
    ERROR: goto ERROR;
}*/

// Scheduler
int num_irqs = 2;
int count = 2;

int irq_enabled[] = {0, 0}; 

void enable_irq(int i) {
  irq_enabled[i] = 1;  
}

void disable_irq(int i) {
  irq_enabled[i] = 0;  
}

void schedule_irq(void) {
  int j = count;
  int irq;

  for (int i = 0; i < j; i++) {
    irq = __VERIFIER_nondet_int();
    __VERIFIER_assume(irq >= 0 && irq < num_irqs);

    if(irq_enabled[irq] && __VERIFIER_nondet_bool()) {
      count--;

      switch(irq) {
        case 0:
          task_communicate();
          break;
        case 1:
          task_measure();
          break;
        default:
          ;
      }
    }
  }
}


void stop(void)
{
  systemState = IDLE;
}

inline void start(uint32_t _startTime,uint16_t _interval)
{
schedule_irq(); 
  if (numberOfRecords+1 >= MAX_RECORDS) { // R-W dependency
    stop();
  }
schedule_irq(); 
  intervalCounter = 0; // W-W dependency
schedule_irq(); 
  tickCounter = 0; // W-W dependency
schedule_irq(); 
  numberOfRecords = 0; // W-W dependency
schedule_irq(); 
  startTime = _startTime;
schedule_irq(); 
  interval = _interval; // W-W dependency
schedule_irq(); 
  systemState = LOGGING;
schedule_irq(); 
  __VERIFIER_assert(numberOfRecords==0);
  //assert(numberOfRecords==0);
  //if(numberOfRecords!=0) irq_err(); // R-W dependency
}

extern uint16_t read_sensor_value();
void task_measure(void) 
{
disable_irq(0);
disable_irq(1);

  tickCounter++;

  if (tickCounter == interval) {
    uint16_t value = read_sensor_value();
 
    tickCounter = 0;
    intervalCounter++;
    
    records[numberOfRecords] =  value;
    numberOfRecords++;
  }
enable_irq(0);
enable_irq(1);
}

extern uint8_t read_cmd();
uint8_t get_cmd(uint8_t* decodingStatus) {
  uint8_t cmd = read_cmd();
  *decodingStatus = CMD_DEC_ERROR;
  if(cmd==CMD_STOP || cmd==CMD_READ_STATE || cmd==CMD_READ_DATA ||
     cmd==CMD_START) {
    *decodingStatus = CMD_DEC_OK;
  }
  return cmd;
}

extern uint16_t get_uint16();
extern uint32_t get_uint32();
extern void send_response();
void task_communicate(void)
{
disable_irq(0);
enable_irq(1);

  uint8_t decodingStatus;
  uint8_t cmd = get_cmd(&decodingStatus);
  uint8_t rspStatus = RSP_SUCCESS;

  if (decodingStatus != CMD_DEC_OK) {
    rspStatus = RSP_ERROR;
  }
  if (rspStatus == RSP_SUCCESS) {
    switch(cmd) {
      case CMD_STOP:   
        if (systemState != LOGGING) {
          rspStatus = RSP_ERROR;
        } else {
          stop();
        }
        send_response();
        break;
      case CMD_READ_STATE:
        send_response();
	break;
      case CMD_READ_DATA:
        send_response();
	break;
      case CMD_START:
        if (systemState != IDLE) {
          rspStatus = RSP_ERROR;  
        } else {
          uint32_t _startTime = get_uint32();
          uint16_t _interval = get_uint16();
	  start(_startTime,_interval);
	}
        send_response();
        break;
      default: break;
    }
  }
enable_irq(0);
}


void initialize(void) {
  systemState = IDLE; // W-W dependency
}

void run(void)
{
// Lihao
enable_irq(0);
enable_irq(1);
schedule_irq(); 

/*
  __CPROVER_ASYNC_1: 
  task_communicate();

  __CPROVER_ASYNC_1: 
  task_measure();
*/

/*
  while(1) {
    uint8_t currentTask;
    switch(currentTask) {
      case TASK_COMMUNICATE :
        task_communicate();
	break;
      case TASK_MEASURE:
        task_measure();
        break;
      default: break;
    }
  }
*/
}

void main() {
    initialize();
    run();
}
